@@ -21,17 +21,24 @@
 
 static void account_iface_init (gpointer, gpointer);
 
-G_DEFINE_TYPE_WITH_CODE (SimpleAccount,
-    simple_account,
+G_DEFINE_TYPE_WITH_CODE (TpTestsSimpleAccount,
+    tp_tests_simple_account,
     G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_ACCOUNT,
         account_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_ACCOUNT_INTERFACE_ADDRESSING,
+        NULL);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_ACCOUNT_INTERFACE_STORAGE,
+        NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
         tp_dbus_properties_mixin_iface_init)
     )
 
 /* TP_IFACE_ACCOUNT is implied */
-static const char *ACCOUNT_INTERFACES[] = { NULL };
+static const char *ACCOUNT_INTERFACES[] = {
+    TP_IFACE_ACCOUNT_INTERFACE_ADDRESSING,
+    TP_IFACE_ACCOUNT_INTERFACE_STORAGE,
+    NULL };
 
 enum
 {
@@ -52,9 +59,14 @@ enum
   PROP_REQUESTED_PRESENCE,
   PROP_NORMALIZED_NAME,
   PROP_HAS_BEEN_ONLINE,
+  PROP_URI_SCHEMES,
+  PROP_STORAGE_PROVIDER,
+  PROP_STORAGE_IDENTIFIER,
+  PROP_STORAGE_SPECIFIC_INFORMATION,
+  PROP_STORAGE_RESTRICTIONS
 };
 
-struct _SimpleAccountPrivate
+struct _TpTestsSimpleAccountPrivate
 {
   gpointer unused;
 };
@@ -64,32 +76,32 @@ account_iface_init (gpointer klass,
     gpointer unused G_GNUC_UNUSED)
 {
 #define IMPLEMENT(x) tp_svc_account_implement_##x (\
-  klass, simple_account_##x)
+  klass, tp_tests_simple_account_##x)
   /* TODO */
 #undef IMPLEMENT
 }
 
 
 static void
-simple_account_init (SimpleAccount *self)
+tp_tests_simple_account_init (TpTestsSimpleAccount *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, SIMPLE_TYPE_ACCOUNT,
-      SimpleAccountPrivate);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, TP_TESTS_TYPE_SIMPLE_ACCOUNT,
+      TpTestsSimpleAccountPrivate);
 }
 
+/* you may have noticed this is not entirely realistic */
+static const gchar * const uri_schemes[] = { "about", "telnet", NULL };
+
 static void
-simple_account_get_property (GObject *object,
+tp_tests_simple_account_get_property (GObject *object,
               guint property_id,
               GValue *value,
               GParamSpec *spec)
 {
-  GValueArray *presence;
+  GValue identifier = { 0, };
 
-  presence = tp_value_array_build (3,
-      G_TYPE_UINT, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE,
-      G_TYPE_STRING, "available",
-      G_TYPE_STRING, "",
-      G_TYPE_INVALID);
+  g_value_init (&identifier, G_TYPE_STRING);
+  g_value_set_string (&identifier, "unique-identifier");
 
   switch (property_id) {
     case PROP_INTERFACES:
@@ -114,7 +126,11 @@ simple_account_get_property (GObject *object,
       g_value_take_boxed (value, g_hash_table_new (NULL, NULL));
       break;
     case PROP_AUTOMATIC_PRESENCE:
-      g_value_set_boxed (value, presence);
+      g_value_take_boxed (value, tp_value_array_build (3,
+            G_TYPE_UINT, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE,
+            G_TYPE_STRING, "automatically-available",
+            G_TYPE_STRING, "this is my AutomaticPresence",
+            G_TYPE_INVALID));
       break;
     case PROP_CONNECT_AUTO:
       g_value_set_boolean (value, FALSE);
@@ -129,23 +145,52 @@ simple_account_get_property (GObject *object,
       g_value_set_uint (value, TP_CONNECTION_STATUS_REASON_REQUESTED);
       break;
     case PROP_CURRENT_PRESENCE:
-      g_value_set_boxed (value, presence);
+      g_value_take_boxed (value, tp_value_array_build (3,
+            G_TYPE_UINT, TP_CONNECTION_PRESENCE_TYPE_AWAY,
+            G_TYPE_STRING, "currently-away",
+            G_TYPE_STRING, "this is my CurrentPresence",
+            G_TYPE_INVALID));
       break;
     case PROP_REQUESTED_PRESENCE:
-      g_value_set_boxed (value, presence);
+      g_value_take_boxed (value, tp_value_array_build (3,
+            G_TYPE_UINT, TP_CONNECTION_PRESENCE_TYPE_BUSY,
+            G_TYPE_STRING, "requesting",
+            G_TYPE_STRING, "this is my RequestedPresence",
+            G_TYPE_INVALID));
       break;
     case PROP_NORMALIZED_NAME:
-      g_value_set_string (value, "");
+      g_value_set_string (value, "bob.mcbadgers@example.com");
       break;
     case PROP_HAS_BEEN_ONLINE:
       g_value_set_boolean (value, TRUE);
       break;
+    case PROP_STORAGE_PROVIDER:
+      g_value_set_string (value, "org.freedesktop.Telepathy.glib.test");
+      break;
+    case PROP_STORAGE_IDENTIFIER:
+      g_value_set_boxed (value, &identifier);
+      break;
+    case PROP_STORAGE_SPECIFIC_INFORMATION:
+      g_value_take_boxed (value, tp_asv_new (
+            "one", G_TYPE_INT, 1,
+            "two", G_TYPE_UINT, 2,
+            "marco", G_TYPE_STRING, "polo",
+            NULL));
+      break;
+    case PROP_STORAGE_RESTRICTIONS:
+      g_value_set_uint (value,
+          TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_ENABLED |
+          TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_PARAMETERS);
+      break;
+    case PROP_URI_SCHEMES:
+      g_value_set_boxed (value, uri_schemes);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, spec);
       break;
   }
 
-  g_boxed_free (TP_STRUCT_TYPE_SIMPLE_PRESENCE, presence);
+  g_value_unset (&identifier);
 }
 
 /**
@@ -154,7 +199,7 @@ simple_account_get_property (GObject *object,
   * Properties.GetAll().
   */
 static void
-simple_account_class_init (SimpleAccountClass *klass)
+tp_tests_simple_account_class_init (TpTestsSimpleAccountClass *klass)
 {
   GObjectClass *object_class = (GObjectClass *) klass;
   GParamSpec *param_spec;
@@ -179,17 +224,42 @@ simple_account_class_init (SimpleAccountClass *klass)
         { NULL }
   };
 
+  static TpDBusPropertiesMixinPropImpl ais_props[] = {
+        { "StorageProvider", "storage-provider", NULL },
+        { "StorageIdentifier", "storage-identifier", NULL },
+        { "StorageSpecificInformation", "storage-specific-information", NULL },
+        { "StorageRestrictions", "storage-restrictions", NULL },
+        { NULL },
+  };
+
+  static TpDBusPropertiesMixinPropImpl aia_props[] = {
+        { "URISchemes", "uri-schemes", NULL },
+        { NULL },
+  };
+
   static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
         { TP_IFACE_ACCOUNT,
           tp_dbus_properties_mixin_getter_gobject_properties,
           NULL,
           a_props
         },
+        {
+          TP_IFACE_ACCOUNT_INTERFACE_STORAGE,
+          tp_dbus_properties_mixin_getter_gobject_properties,
+          NULL,
+          ais_props
+        },
+        {
+          TP_IFACE_ACCOUNT_INTERFACE_ADDRESSING,
+          tp_dbus_properties_mixin_getter_gobject_properties,
+          NULL,
+          aia_props
+        },
         { NULL },
   };
 
-  g_type_class_add_private (klass, sizeof (SimpleAccountPrivate));
-  object_class->get_property = simple_account_get_property;
+  g_type_class_add_private (klass, sizeof (TpTestsSimpleAccountPrivate));
+  object_class->get_property = tp_tests_simple_account_get_property;
 
   param_spec = g_param_spec_boxed ("interfaces", "Extra D-Bus interfaces",
       "In this case we only implement Account, so none.",
@@ -295,7 +365,41 @@ simple_account_class_init (SimpleAccountClass *klass)
   g_object_class_install_property (object_class, PROP_HAS_BEEN_ONLINE,
       param_spec);
 
+  param_spec = g_param_spec_string ("storage-provider", "storage provider",
+      "StorageProvider property",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_STORAGE_PROVIDER,
+      param_spec);
+
+  param_spec = g_param_spec_boxed ("storage-identifier", "storage identifier",
+      "StorageIdentifier property",
+      G_TYPE_VALUE,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_STORAGE_IDENTIFIER,
+      param_spec);
+
+  param_spec = g_param_spec_boxed ("storage-specific-information",
+      "storage specific information", "StorageSpecificInformation property",
+      TP_HASH_TYPE_STRING_VARIANT_MAP,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_STORAGE_SPECIFIC_INFORMATION, param_spec);
+
+  param_spec = g_param_spec_uint ("storage-restrictions",
+      "storage restrictions", "StorageRestrictions property",
+      0, G_MAXUINT, 0,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_STORAGE_RESTRICTIONS,
+      param_spec);
+
+  param_spec = g_param_spec_boxed ("uri-schemes", "URI schemes",
+      "Some URI schemes",
+      G_TYPE_STRV,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_URI_SCHEMES, param_spec);
+
   klass->dbus_props_class.interfaces = prop_interfaces;
   tp_dbus_properties_mixin_class_init (object_class,
-      G_STRUCT_OFFSET (SimpleAccountClass, dbus_props_class));
+      G_STRUCT_OFFSET (TpTestsSimpleAccountClass, dbus_props_class));
 }
